@@ -25,17 +25,19 @@ from marshmallow_sqlalchemy import SQLAlchemySchema, auto_field
 from airflow.api_connexion.parameters import validate_istimezone
 from airflow.api_connexion.schemas.common_schema import JsonObjectField
 from airflow.api_connexion.schemas.enum_schemas import TaskInstanceStateField
+from airflow.api_connexion.schemas.job_schema import JobSchema
 from airflow.api_connexion.schemas.sla_miss_schema import SlaMissSchema
+from airflow.api_connexion.schemas.trigger_schema import TriggerSchema
 from airflow.models import SlaMiss, TaskInstance
 from airflow.utils.helpers import exactly_one
-from airflow.utils.state import State
+from airflow.utils.state import TaskInstanceState
 
 
 class TaskInstanceSchema(SQLAlchemySchema):
-    """Task instance schema"""
+    """Task instance schema."""
 
     class Meta:
-        """Meta"""
+        """Meta."""
 
         model = TaskInstance
 
@@ -60,8 +62,11 @@ class TaskInstanceSchema(SQLAlchemySchema):
     queued_dttm = auto_field(data_key="queued_when")
     pid = auto_field()
     executor_config = auto_field()
+    note = auto_field()
     sla_miss = fields.Nested(SlaMissSchema, dump_default=None)
     rendered_fields = JsonObjectField(dump_default={})
+    trigger = fields.Nested(TriggerSchema)
+    triggerer_job = fields.Nested(JobSchema)
 
     def get_attribute(self, obj, attr, default):
         if attr == "sla_miss":
@@ -76,25 +81,27 @@ class TaskInstanceSchema(SQLAlchemySchema):
 
 
 class TaskInstanceCollection(NamedTuple):
-    """List of task instances with metadata"""
+    """List of task instances with metadata."""
 
     task_instances: list[tuple[TaskInstance, SlaMiss | None]]
     total_entries: int
 
 
 class TaskInstanceCollectionSchema(Schema):
-    """Task instance collection schema"""
+    """Task instance collection schema."""
 
     task_instances = fields.List(fields.Nested(TaskInstanceSchema))
     total_entries = fields.Int()
 
 
 class TaskInstanceBatchFormSchema(Schema):
-    """Schema for the request form passed to Task Instance Batch endpoint"""
+    """Schema for the request form passed to Task Instance Batch endpoint."""
 
     page_offset = fields.Int(load_default=0, validate=validate.Range(min=0))
     page_limit = fields.Int(load_default=100, validate=validate.Range(min=1))
     dag_ids = fields.List(fields.Str(), load_default=None)
+    dag_run_ids = fields.List(fields.Str(), load_default=None)
+    task_ids = fields.List(fields.Str(), load_default=None)
     execution_date_gte = fields.DateTime(load_default=None, validate=validate_istimezone)
     execution_date_lte = fields.DateTime(load_default=None, validate=validate_istimezone)
     start_date_gte = fields.DateTime(load_default=None, validate=validate_istimezone)
@@ -103,13 +110,13 @@ class TaskInstanceBatchFormSchema(Schema):
     end_date_lte = fields.DateTime(load_default=None, validate=validate_istimezone)
     duration_gte = fields.Int(load_default=None)
     duration_lte = fields.Int(load_default=None)
-    state = fields.List(fields.Str(), load_default=None)
+    state = fields.List(fields.Str(allow_none=True), load_default=None)
     pool = fields.List(fields.Str(), load_default=None)
     queue = fields.List(fields.Str(), load_default=None)
 
 
 class ClearTaskInstanceFormSchema(Schema):
-    """Schema for handling the request of clearing task instance of a Dag"""
+    """Schema for handling the request of clearing task instance of a Dag."""
 
     dry_run = fields.Boolean(load_default=True)
     start_date = fields.DateTime(load_default=None, validate=validate_istimezone)
@@ -128,7 +135,7 @@ class ClearTaskInstanceFormSchema(Schema):
 
     @validates_schema
     def validate_form(self, data, **kwargs):
-        """Validates clear task instance form"""
+        """Validate clear task instance form."""
         if data["only_failed"] and data["only_running"]:
             raise ValidationError("only_failed and only_running both are set to True")
         if data["start_date"] and data["end_date"]:
@@ -143,7 +150,7 @@ class ClearTaskInstanceFormSchema(Schema):
 
 
 class SetTaskInstanceStateFormSchema(Schema):
-    """Schema for handling the request of setting state of task instance of a DAG"""
+    """Schema for handling the request of setting state of task instance of a DAG."""
 
     dry_run = fields.Boolean(dump_default=True)
     task_id = fields.Str(required=True)
@@ -153,17 +160,34 @@ class SetTaskInstanceStateFormSchema(Schema):
     include_downstream = fields.Boolean(required=True)
     include_future = fields.Boolean(required=True)
     include_past = fields.Boolean(required=True)
-    new_state = TaskInstanceStateField(required=True, validate=validate.OneOf([State.SUCCESS, State.FAILED]))
+    new_state = TaskInstanceStateField(
+        required=True,
+        validate=validate.OneOf(
+            [TaskInstanceState.SUCCESS, TaskInstanceState.FAILED, TaskInstanceState.SKIPPED]
+        ),
+    )
 
     @validates_schema
     def validate_form(self, data, **kwargs):
-        """Validates set task instance state form"""
+        """Validate set task instance state form."""
         if not exactly_one(data.get("execution_date"), data.get("dag_run_id")):
             raise ValidationError("Exactly one of execution_date or dag_run_id must be provided")
 
 
+class SetSingleTaskInstanceStateFormSchema(Schema):
+    """Schema for handling the request of updating state of a single task instance."""
+
+    dry_run = fields.Boolean(load_default=True)
+    new_state = TaskInstanceStateField(
+        required=True,
+        validate=validate.OneOf(
+            [TaskInstanceState.SUCCESS, TaskInstanceState.FAILED, TaskInstanceState.SKIPPED]
+        ),
+    )
+
+
 class TaskInstanceReferenceSchema(Schema):
-    """Schema for the task instance reference schema"""
+    """Schema for the task instance reference schema."""
 
     task_id = fields.Str()
     run_id = fields.Str(data_key="dag_run_id")
@@ -172,21 +196,31 @@ class TaskInstanceReferenceSchema(Schema):
 
 
 class TaskInstanceReferenceCollection(NamedTuple):
-    """List of objects with metadata about taskinstance and dag_run_id"""
+    """List of objects with metadata about taskinstance and dag_run_id."""
 
     task_instances: list[tuple[TaskInstance, str]]
 
 
 class TaskInstanceReferenceCollectionSchema(Schema):
-    """Collection schema for task reference"""
+    """Collection schema for task reference."""
 
     task_instances = fields.List(fields.Nested(TaskInstanceReferenceSchema))
 
 
+class SetTaskInstanceNoteFormSchema(Schema):
+    """Schema for settings a note for a TaskInstance."""
+
+    # Note: We can't add map_index to the url as subpaths can't start with dashes.
+    map_index = fields.Int(allow_none=False)
+    note = fields.String(allow_none=True, validate=validate.Length(max=1000))
+
+
 task_instance_schema = TaskInstanceSchema()
 task_instance_collection_schema = TaskInstanceCollectionSchema()
 task_instance_batch_form = TaskInstanceBatchFormSchema()
 clear_task_instance_form = ClearTaskInstanceFormSchema()
 set_task_instance_state_form = SetTaskInstanceStateFormSchema()
+set_single_task_instance_state_form = SetSingleTaskInstanceStateFormSchema()
 task_instance_reference_schema = TaskInstanceReferenceSchema()
 task_instance_reference_collection_schema = TaskInstanceReferenceCollectionSchema()
+set_task_instance_note_form_schema = SetTaskInstanceNoteFormSchema()
