@@ -19,23 +19,24 @@ from __future__ import annotations
 
 import warnings
 from datetime import timedelta
-from tempfile import gettempdir
 
 from flask import Flask
 from flask_appbuilder import SQLA
-from flask_caching import Cache
 from flask_wtf.csrf import CSRFProtect
 from markupsafe import Markup
 from sqlalchemy.engine.url import make_url
 
 from airflow import settings
+from airflow.api_internal.internal_api_call import InternalApiConfig
 from airflow.configuration import conf
 from airflow.exceptions import AirflowConfigException, RemovedInAirflow3Warning
 from airflow.logging_config import configure_logging
 from airflow.models import import_all_models
+from airflow.settings import _ENABLE_AIP_44
 from airflow.utils.json import AirflowJsonProvider
 from airflow.www.extensions.init_appbuilder import init_appbuilder
 from airflow.www.extensions.init_appbuilder_links import init_appbuilder_links
+from airflow.www.extensions.init_cache import init_cache
 from airflow.www.extensions.init_dagbag import init_dagbag
 from airflow.www.extensions.init_jinja_globals import init_jinja_globals
 from airflow.www.extensions.init_manifest_files import configure_manifest_files
@@ -49,8 +50,8 @@ from airflow.www.extensions.init_session import init_airflow_session_interface
 from airflow.www.extensions.init_views import (
     init_api_connexion,
     init_api_experimental,
+    init_api_internal,
     init_appbuilder_views,
-    init_connection_form,
     init_error_handlers,
     init_flash_views,
     init_plugins,
@@ -65,7 +66,7 @@ csrf = CSRFProtect()
 
 
 def sync_appbuilder_roles(flask_app):
-    """Sync appbuilder roles to DB"""
+    """Sync appbuilder roles to DB."""
     # Garbage collect old permissions/views after they have been modified.
     # Otherwise, when the name of a view or menu is changed, the framework
     # will add the new Views and Menus names to the backend, but will not
@@ -75,12 +76,17 @@ def sync_appbuilder_roles(flask_app):
 
 
 def create_app(config=None, testing=False):
-    """Create a new instance of Airflow WWW app"""
+    """Create a new instance of Airflow WWW app."""
     flask_app = Flask(__name__)
     flask_app.secret_key = conf.get("webserver", "SECRET_KEY")
 
     flask_app.config["PERMANENT_SESSION_LIFETIME"] = timedelta(minutes=settings.get_session_lifetime_config())
-    flask_app.config.from_pyfile(settings.WEBSERVER_CONFIG, silent=True)
+
+    webserver_config = conf.get_mandatory_value("webserver", "config_file")
+    # Enable customizations in webserver_config.py to be applied via Flask.current_app.
+    with flask_app.app_context():
+        flask_app.config.from_pyfile(webserver_config, silent=True)
+
     flask_app.config["TESTING"] = testing
     flask_app.config["SQLALCHEMY_DATABASE_URI"] = conf.get("database", "SQL_ALCHEMY_CONN")
 
@@ -125,6 +131,8 @@ def create_app(config=None, testing=False):
     flask_app.json_provider_class = AirflowJsonProvider
     flask_app.json = AirflowJsonProvider(flask_app)
 
+    InternalApiConfig.force_database_direct_access()
+
     csrf.init_app(flask_app)
 
     init_wsgi_middleware(flask_app)
@@ -139,8 +147,7 @@ def create_app(config=None, testing=False):
 
     init_robots(flask_app)
 
-    cache_config = {"CACHE_TYPE": "flask_caching.backends.filesystem", "CACHE_DIR": gettempdir()}
-    Cache(app=flask_app, config=cache_config)
+    init_cache(flask_app)
 
     init_flash_views(flask_app)
 
@@ -155,9 +162,12 @@ def create_app(config=None, testing=False):
         init_appbuilder_views(flask_app)
         init_appbuilder_links(flask_app)
         init_plugins(flask_app)
-        init_connection_form()
         init_error_handlers(flask_app)
         init_api_connexion(flask_app)
+        if conf.getboolean("webserver", "run_internal_api", fallback=False):
+            if not _ENABLE_AIP_44:
+                raise RuntimeError("The AIP_44 is not enabled so you cannot use it.")
+            init_api_internal(flask_app)
         init_api_experimental(flask_app)
 
         sync_appbuilder_roles(flask_app)
@@ -170,7 +180,7 @@ def create_app(config=None, testing=False):
 
 
 def cached_app(config=None, testing=False):
-    """Return cached instance of Airflow WWW app"""
+    """Return cached instance of Airflow WWW app."""
     global app
     if not app:
         app = create_app(config=config, testing=testing)
