@@ -17,36 +17,41 @@
 from __future__ import annotations
 
 import logging
+import os
 import warnings
-from os import path
+from functools import cached_property
+from pathlib import Path
+from typing import TYPE_CHECKING
 
 from connexion import FlaskApi, ProblemException, Resolver
 from connexion.decorators.validation import RequestBodyValidator
 from connexion.exceptions import BadRequestProblem
-from flask import Flask, request
+from flask import request
 
 from airflow.api_connexion.exceptions import common_error_handler
-from airflow.compat.functools import cached_property
 from airflow.configuration import conf
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.security import permissions
 from airflow.utils.yaml import safe_load
 
+if TYPE_CHECKING:
+    from flask import Flask
+
 log = logging.getLogger(__name__)
 
 # airflow/www/extensions/init_views.py => airflow/
-ROOT_APP_DIR = path.abspath(path.join(path.dirname(__file__), path.pardir, path.pardir))
+ROOT_APP_DIR = Path(__file__).parents[2].resolve()
 
 
 def init_flash_views(app):
-    """Init main app view - redirect to FAB"""
+    """Init main app view - redirect to FAB."""
     from airflow.www.blueprints import routes
 
     app.register_blueprint(routes)
 
 
 def init_appbuilder_views(app):
-    """Initialize Web UI views"""
+    """Initialize Web UI views."""
     from airflow.models import import_all_models
 
     import_all_models()
@@ -125,7 +130,7 @@ def init_appbuilder_views(app):
 
 
 def init_plugins(app):
-    """Integrate Flask and FAB with plugins"""
+    """Integrate Flask and FAB with plugins."""
     from airflow import plugins_manager
 
     plugins_manager.initialize_web_ui_plugins()
@@ -154,7 +159,7 @@ def init_plugins(app):
 
 
 def init_error_handlers(app: Flask):
-    """Add custom errors handlers"""
+    """Add custom errors handlers."""
     from airflow.www import views
 
     app.register_error_handler(500, views.show_traceback)
@@ -162,7 +167,7 @@ def init_error_handlers(app: Flask):
 
 
 def set_cors_headers_on_response(response):
-    """Add response headers"""
+    """Add response headers."""
     allow_headers = conf.get("api", "access_control_allow_headers")
     allow_methods = conf.get("api", "access_control_allow_methods")
     allow_origins = conf.get("api", "access_control_allow_origins")
@@ -226,7 +231,7 @@ class _CustomErrorRequestBodyValidator(RequestBodyValidator):
 
 
 def init_api_connexion(app: Flask) -> None:
-    """Initialize Stable API"""
+    """Initialize Stable API."""
     base_path = "/api/v1"
 
     from airflow.www import views
@@ -249,7 +254,7 @@ def init_api_connexion(app: Flask) -> None:
         else:
             return views.method_not_allowed(ex)
 
-    with open(path.join(ROOT_APP_DIR, "api_connexion", "openapi", "v1.yaml")) as f:
+    with ROOT_APP_DIR.joinpath("api_connexion", "openapi", "v1.yaml").open() as f:
         specification = safe_load(f)
     api_bp = FlaskApi(
         specification=specification,
@@ -257,7 +262,7 @@ def init_api_connexion(app: Flask) -> None:
         base_path=base_path,
         options={
             "swagger_ui": conf.getboolean("webserver", "enable_swagger_ui", fallback=True),
-            "swagger_path": path.join(ROOT_APP_DIR, "www", "static", "dist", "swagger-ui"),
+            "swagger_path": os.fspath(ROOT_APP_DIR.joinpath("www", "static", "dist", "swagger-ui")),
         },
         strict_validation=True,
         validate_responses=True,
@@ -271,11 +276,11 @@ def init_api_connexion(app: Flask) -> None:
 
 
 def init_api_internal(app: Flask, standalone_api: bool = False) -> None:
-    """Initialize Internal API"""
+    """Initialize Internal API."""
     if not standalone_api and not conf.getboolean("webserver", "run_internal_api", fallback=False):
         return
 
-    with open(path.join(ROOT_APP_DIR, "api_internal", "openapi", "internal_api_v1.yaml")) as f:
+    with ROOT_APP_DIR.joinpath("api_internal", "openapi", "internal_api_v1.yaml").open() as f:
         specification = safe_load(f)
     api_bp = FlaskApi(
         specification=specification,
@@ -292,7 +297,7 @@ def init_api_internal(app: Flask, standalone_api: bool = False) -> None:
 
 
 def init_api_experimental(app):
-    """Initialize Experimental API"""
+    """Initialize Experimental API."""
     if not conf.getboolean("api", "enable_experimental_api", fallback=False):
         return
     from airflow.www.api.experimental import endpoints
